import sys
import os
modules_path = '../../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

import json
from datetime import datetime, date
from db.repository import Repository
from db.entities.timeslice import TimeSlice
from db.entities import ClusterSet
from typing import Tuple, Dict, Any

TimeSliceKey = Tuple[int, int]

def convert_to_time_slice_key(timestamp: str) -> TimeSliceKey:
    '''Returns the tuple (year, week_of_year) from a timestamp. This is used as the key for the slicing.'''
    time = datetime.utcfromtimestamp(float(timestamp[0:10]))
    (y, w, _) = time.isocalendar()
    return (y, w)


def split_clusterset_by_time(clustersets) -> Dict[TimeSliceKey, TimeSlice]:
    '''
    Distributes all nodes of a single clusterset into individual time slices based on their timestamps. 
    If a node spans over multiple slices it will be added to all of them.
    Information about clusters and the nodes in the clusters will not be changed.

    :params clustersets: The clusterset whichs nodes are split
    :returns: A dict of time slices where the key is the time info and value is the information about the time slice
    '''

    time_slices: Dict[Any, TimeSlice] = {} 
    for cluster_no in clusterset.clusters:
        for node in cluster_no.nodes:

            time_keys = {
                convert_to_time_slice_key(str(node['Finished_time'])), 
                convert_to_time_slice_key(str(node['Starting_time']))
            }

            for time_key in time_keys:
                if time_key not in time_slices:
                    time_slices[time_key] = TimeSlice(time_key, clusterset.layer_name)

                time_slices[time_key].add_node_to_cluster(cluster_no.cluster_label, node)

    return time_slices
        

if __name__ == "__main__":
    repo = Repository()

    repo.remove_all_time_slices()

    clustersets = repo.get_clustersets()
    for clusterset in clustersets:
        time_slices = split_clusterset_by_time(clusterset)

        for k,v in time_slices.items():
            repo.add_time_slice(v)
